@@ -52,7 +52,7 @@
 --
 -- To create 'Generic' instances for @A@ and @B@ via "GHC.Generics", we say
 --
--- > {-# LANGUAGE DeriveGenerics #-}
+-- > {-# LANGUAGE DeriveGeneric #-}
 -- >
 -- > import qualified GHC.Generics as GHC
 -- > import Generics.SOP
@@ -221,6 +221,7 @@ module Generics.SOP (
     -- * n-ary datatypes
   , NP(..)
   , NS(..)
+  , unZ
   , SOP(..)
   , unSOP
   , POP(..)
@@ -239,8 +240,14 @@ module Generics.SOP (
     -- * Combinators
     -- ** Constructing products
   , HPure(..)
+    -- ** Destructing products
+  , hd
+  , tl
+  , Projection
+  , projections
+  , shiftProjection
     -- ** Application
-  , (-.->)(..)
+  , type (-.->)(..)
   , fn
   , fn_2
   , fn_3
@@ -264,6 +271,7 @@ module Generics.SOP (
   , Injection
   , injections
   , shift
+  , shiftInjection
   , apInjs_NP
   , apInjs_POP
     -- ** Dealing with @'All' c@
@@ -297,6 +305,7 @@ module Generics.SOP (
     -- ** Singletons
   , SList(..)
   , SListI(..)
+  , SListI2
   , Sing
   , SingI(..)
     -- *** Shape of type-level lists
