@@ -2,6 +2,9 @@
 #if __GLASGOW_HASKELL__ < 710
 {-# LANGUAGE OverlappingInstances #-}
 #endif
+#if __GLASGOW_HASKELL__ >= 800
+{-# LANGUAGE UndecidableSuperClasses #-}
+#endif
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-deprecations #-}
 -- | Constraints for indexed datatypes.
 --
@@ -44,8 +47,8 @@ instance (AllF f xs, SListI xs) => All f xs
 -- | Type family used to implement 'All'.
 --
 type family AllF (c :: k -> Constraint) (xs :: [k]) :: Constraint
-type instance AllF c '[]       = ()
-type instance AllF c (x ': xs) = (c x, All c xs)
+type instance AllF _c '[]       = ()
+type instance AllF  c (x ': xs) = (c x, All c xs)
 
 -- | Require a singleton for every inner list in a list of lists.
 type SListI2 = All SListI
@@ -95,18 +98,24 @@ instance (AllF (All f) xss, SListI xss) => All2 f xss
 -- 'NS'. For example, in order to denote that all elements on an
 -- @'NP' f xs@ satisfy 'Show', we can say @'All' ('Show' :. f) xs@.
 --
+-- @since 0.2
+--
 class (f (g x)) => (f `Compose` g) x
 instance (f (g x)) => (f `Compose` g) x
 infixr 9 `Compose`
 
 -- | Pairing of constraints.
 --
+-- @since 0.2
+--
 class (f x, g x) => (f `And` g) x
 instance (f x, g x) => (f `And` g) x
 infixl 7 `And`
 
 -- | A constraint that can always be satisfied.
 --
+-- @since 0.2
+--
 class Top x
 instance Top x
 
@@ -135,5 +144,5 @@ instance
 #if __GLASGOW_HASKELL__ >= 710
   {-# OVERLAPPING #-}
 #endif
-  All SListI xss => SingI (xss :: [[k]]) where
+  (All SListI xss, SListI xss) => SingI (xss :: [[k]]) where
   sing = sList
