@@ -15,6 +15,12 @@ module Generics.SOP.NP
     -- * Application
   , ap_NP
   , ap_POP
+    -- * Destructing products
+  , hd
+  , tl
+  , Projection
+  , projections
+  , shiftProjection
     -- * Lifting / mapping
   , liftA_NP
   , liftA_POP
@@ -219,7 +225,9 @@ fromList = go sList
 ap_NP :: NP (f -.-> g) xs -> NP f xs -> NP g xs
 ap_NP Nil           Nil        = Nil
 ap_NP (Fn f :* fs)  (x :* xs)  = f x :* ap_NP fs xs
+#if __GLASGOW_HASKELL__ < 800
 ap_NP _ _ = error "inaccessible"
+#endif
 
 -- | Specialization of 'hap'.
 --
@@ -232,7 +240,9 @@ ap_POP (POP fss') (POP xss') = POP (go fss' xss')
     go :: NP (NP (f -.-> g)) xss -> NP (NP f) xss -> NP (NP g) xss
     go Nil         Nil         = Nil
     go (fs :* fss) (xs :* xss) = ap_NP fs xs :* go fss xss
+#if __GLASGOW_HASKELL__ < 800
     go _           _           = error "inaccessible"
+#endif
 
 -- The definition of 'ap_POP' is a more direct variant of
 -- '_ap_POP_spec'. The direct definition has the advantage
@@ -246,6 +256,38 @@ type instance Prod POP = POP
 instance HAp NP  where hap = ap_NP
 instance HAp POP where hap = ap_POP
 
+-- * Destructing products
+
+-- | Obtain the head of an n-ary product.
+--
+-- @since 0.2.1.0
+--
+hd :: NP f (x ': xs) -> f x
+hd (x :* _xs) = x
+
+-- | Obtain the tail of an n-ary product.
+--
+-- @since 0.2.1.0
+--
+tl :: NP f (x ': xs) -> NP f xs
+tl (_x :* xs) = xs
+
+-- | The type of projections from an n-ary product.
+--
+type Projection (f :: k -> *) (xs :: [k]) = K (NP f xs) -.-> f
+
+-- | Compute all projections from an n-ary product.
+--
+-- Each element of the resulting product contains one of the projections.
+--
+projections :: forall xs f . SListI xs => NP (Projection f xs) xs
+projections = case sList :: SList xs of
+  SNil  -> Nil
+  SCons -> fn (hd . unK) :* liftA_NP shiftProjection projections
+
+shiftProjection :: Projection f xs a -> Projection f (x ': xs) a
+shiftProjection (Fn f) = Fn $ f . K . tl . unK
+
 -- * Lifting / mapping
 
 -- | Specialization of 'hliftA'.
@@ -364,7 +406,7 @@ czipWith3_POP = hczipWith3
 -- 'hcliftA'' :: 'All2' c xss => proxy c -> (forall xs. 'All' c xs => f xs -> f' xs) -> 'Generics.SOP.NS.NS' f xss -> 'Generics.SOP.NS.NS' f' xss
 -- @
 --
-{-# DEPRECATED hcliftA' "Use 'hclift' or 'hcmap' instead." #-}
+{-# DEPRECATED hcliftA' "Use 'hcliftA' or 'hcmap' instead." #-}
 hcliftA'  :: (All2 c xss, Prod h ~ NP, HAp h) => proxy c -> (forall xs. All c xs => f xs -> f' xs)                                                       -> h f   xss -> h f'   xss
 
 -- | Like 'hcliftA'', but for binary functions.
