@@ -5,11 +5,13 @@ module Generics.SOP.NS
   ( -- * Datatypes
     NS(..)
   , SOP(..)
+  , unZ
   , unSOP
     -- * Constructing sums
   , Injection
   , injections
   , shift
+  , shiftInjection
   , apInjs_NP
   , apInjs_POP
     -- * Application
@@ -100,6 +102,22 @@ deriving instance All (Show `Compose` f) xs => Show (NS f xs)
 deriving instance All (Eq   `Compose` f) xs => Eq   (NS f xs)
 deriving instance (All (Eq `Compose` f) xs, All (Ord `Compose` f) xs) => Ord (NS f xs)
 
+-- | Extract the payload from a unary sum.
+--
+-- For larger sums, this function would be partial, so it is only
+-- provided with a rather restrictive type.
+--
+-- /Example:/
+--
+-- >>> unZ (Z (I 'x'))
+-- I 'x'
+--
+-- @since 0.2.2.0
+--
+unZ :: NS f '[x] -> f x
+unZ (Z x) = x
+unZ _     = error "inaccessible" -- needed even in GHC 8.0.1
+
 -- | A sum of products.
 --
 -- This is a 'newtype' for an 'NS' of an 'NP'. The elements of the
@@ -143,14 +161,22 @@ type Injection (f :: k -> *) (xs :: [k]) = f -.-> K (NS f xs)
 injections :: forall xs f. SListI xs => NP (Injection f xs) xs
 injections = case sList :: SList xs of
   SNil   -> Nil
-  SCons  -> fn (K . Z) :* liftA_NP shift injections
+  SCons  -> fn (K . Z) :* liftA_NP shiftInjection injections
 
+-- | Shift an injection.
+--
+-- Given an injection, return an injection into a sum that is one component larger.
+--
+shiftInjection :: Injection f xs a -> Injection f (x ': xs) a
+shiftInjection (Fn f) = Fn $ K . S . unK . f
+
+{-# DEPRECATED shift "Use 'shiftInjection' instead." #-}
 -- | Shift an injection.
 --
 -- Given an injection, return an injection into a sum that is one component larger.
 --
 shift :: Injection f xs a -> Injection f (x ': xs) a
-shift (Fn f) = Fn $ K . S . unK . f
+shift = shiftInjection
 
 -- | Apply injections to a product.
 --
